% WIP: calculate the degree of orieantation and export to paraview

%*-------------------------------------------------------------------------------------*
%|                                                                                     |
%|  Except where otherwise noted, this work is licensed under a                        |
%|  Creative Commons Attribution-NonCommercial-ShareAlike 4.0                          |
%|  International (CC BY-NC-SA 4.0) license.                                           |
%|                                                                                     |
%|  Copyright (c) 2017 by Paul Scherrer Institute (http://www.psi.ch)                  |
%|                                                                                     |
%|      Author: CXS group, PSI                                                         |
%*------------------------------------------------------------------------------------*
% You may use this code with the following provisions:
%
% If this code, or subfunctions or parts of it, is used for research in a
%   publication or if it is fully or partially rewritten for another
%   computing language the authors and institution should be acknowledged
%   in written form and additionally you should cite:
%     M. Liebi, M. Georgiadis, A. Menzel, P. Schneider, J. Kohlbrecher,
%     O. Bunk, and M. Guizar-Sicairos, “Nanostructure surveys of
%     macroscopic specimens by small-angle scattering tensor tomography,”
%     Nature 527, 349-352 (2015).   (doi:10.1038/nature16056)
%
% A publication that focuses on describing features, or parameters, that
%    are already existing in the code should be first discussed with the
%    authors.
%   
% This code and subroutines are part of a continuous development, they
%    are provided “as they are” without guarantees or liability on part
%    of PSI or the authors. It is the user responsibility to ensure its
%    proper use and the correctness of the results.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Step 4.1 load the data and check the optimization
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% EDIT:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
base_path = '~/Data10';                    % = '~/Data10/' for online analysis,
                                            % provide the path for offline analysis
                                            % Ex: '/das/work/p16/p16649/'
sample_name = 'sample_name';                % name given in the saxs_caller_template
add_name = 'ID';       % additional name the optimizations: = [ ] if not needed
which_coeff = 2;                % which coefficient to show a0 = 1, a2 = 2, a4 = 3, a6 = 4
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

filename = fullfile(base_path,sprintf('analysis/SASTT/%s/projection_data/SASTT_%s_aligned_ASTRA.mat', ...
    sample_name, sample_name));
orig_data = load(filename);
% load the optimization results from SH
filename = fullfile(base_path, sprintf('analysis/SASTT/%s/SH/%s/results/', sample_name, add_name));
        filename = fullfile(filename, sprintf('result_%s_q%d-%d_all_again_%s.mat', sample_name, orig_data.projection(1).par.r_sum{1}(1), ...
            orig_data.projection(1).par.r_sum{1}(end), add_name));
clear orig_data
load(filename);
% info about the spherical harmonics
l = params.l;
m = params.m;
order = length(l);

% to visualize the coefficients
view3(reshape(a_out1((params.numOfvoxels*(which_coeff-1))+1:params.numOfvoxels*which_coeff),...
    params.points_per_line, params.lines, params.lines).*params.mask3D);

%% STEP 4.2 calculate degree of orientation
% apply mask
mask3D = params.mask3D(:)';
% Mask the results
a_1 = a_out1(1:params.numOfvoxels).*mask3D;
a_2 = a_out1(params.numOfvoxels+1:2*params.numOfvoxels).*mask3D;
a_3 = a_out1(2*params.numOfvoxels+1:3*params.numOfvoxels).*mask3D;
if order == 4
    a_4 = a_out1(3*params.numOfvoxels+1:4*params.numOfvoxels).*mask3D;
end

% calculate the degree of orientation
degorientation = zeros(1, numel(params.mask3D));
if order == 4
    degorientation = (abs(a_2.^2) + abs(a_3.^2) + abs(a_4.^2))./...
        (abs(a_1.^2) + abs(a_2.^2) + abs(a_3.^2) + abs(a_4.^2));
elseif order == 3
    degorientation = (abs(a_2.^2) + abs(a_3.^2))./(abs(a_1.^2) + abs(a_2.^2) + abs(a_3.^2));
else
    fprintf('adjust script for right order \n');
end
degree_of_orientation = degorientation;
degorientation = [];
degree_of_orientation_tomo = reshape(degree_of_orientation, params.points_per_line, ...
    params.lines, params.lines);
view3(degree_of_orientation_tomo.*params.mask3D);
% free memory
clear a_1 a_2 a_3 a_4 mask3D 

%% Step 4.3 check the result of the degree of orientation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% EDIT:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
apply_threshold = 1; % = 0 for applying no threeshowld to the degree of orientation
threshold_max = 0.5; % max degree of orientation
threshold_min = 0; % min degree of orientation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%apply thresholds to the degree of orientation
if (apply_threshold)
    degorientation_threshold = degree_of_orientation;
    degorientation_threshold(degorientation_threshold > threshold_max) = 0;
    degorientation_threshold(degorientation_threshold < threshold_min) = 0;
    degorientation_threshold_tomo = reshape(degorientation_threshold, params.points_per_line,...
        params.lines, params.lines);
    view3(degorientation_threshold_tomo.*params.mask3D);
    degree_of_orientation = degorientation_threshold;
    clear degorientation_threshold degorientation_threshold_tomo
end

%% Step 4.4 normalize the degree of orientation by maximum
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% EDIT:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
normalize_by_max = 1; % true for normalizing the data by maximum degree of orientation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if normalize_by_max
    degorientation_norm = degree_of_orientation./max(degree_of_orientation(:));
    degorientation_norm_tomo = reshape(degorientation_norm, params.points_per_line,...
        params.lines, params.lines);
    view3(degorientation_norm_tomo.*params.mask3D);
    degree_of_orientation = degorientation_norm;
    % free memory for fid
    clear degorientation_norm degorientation_norm_tomo
end

%% Step 4.5 Export data for paraview
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% EDIT:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
% This will export vtk files of orientation vectors to be visualized in
% paraview.
magnitude_symIntensity = 1; % If to use symmetric intensity a0 as vector magnitude.
% This should be =1 in most cases.
magnitude_degOrientation = 0; % If to scale the vectors with degree of orientation.
% This depends on the dataset.
thres_low = 2.5;
thres_high = inf;
% Thresholds for magnitude of final output vectors, can be checked in paraview.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% transform into vector data
xx_tomo = sin(theta_out1).*cos(phi_out1);
yy_tomo = sin(theta_out1).*sin(phi_out1);
zz_tomo = cos(theta_out1);

if magnitude_symIntensity
    xx_tomo = xx_tomo.* a_out1(1:params.numOfvoxels);
    yy_tomo = yy_tomo.* a_out1(1:params.numOfvoxels);
    zz_tomo = zz_tomo.* a_out1(1:params.numOfvoxels);
end

if magnitude_degOrientation
    xx_tomo = xx_tomo.* degree_of_orientation;
    yy_tomo = yy_tomo.* degree_of_orientation;
    zz_tomo = zz_tomo.* degree_of_orientation;
end

    
xx_tomo = reshape(xx_tomo, params.points_per_line, params.lines, params.lines);
yy_tomo = reshape(yy_tomo, params.points_per_line, params.lines, params.lines);
zz_tomo = reshape(zz_tomo, params.points_per_line, params.lines, params.lines);
data_paraview = cat(4, yy_tomo, xx_tomo, zz_tomo);
data_paraview (isnan(data_paraview)) = 0;

%% export as vtk file
for ii=1:params.points_per_line
    for jj=1:params.lines
        for kk=1:params.lines
            if (norm(squeeze(data_paraview(ii,jj,kk,:)))<thres_low)||(norm(squeeze(data_paraview(ii,jj,kk,:)))>thres_high)
                data_paraview(ii,jj,kk,:)=0;
            end
        end
    end
end

vtk_filename = sprintf('%sanalysis/SASTT/%s/SH/optimization_output/%s/SH_paraview_data_%s_%s.vtk', base_path, sample_name,...
    add_name, sample_name, add_name);
vtkwrite(vtk_filename, 1, 'VECTORS',sample_name,data_paraview);

%% alternative to export as raw file
%concatenate matrices
data_paraview = cat(4, xx_tomo, yy_tomo, zz_tomo);
data_paraview = permute(data_paraview, [4 2 1 3]);

% save data
filename = fullfile(params.optimization_output_path, sprintf ('paraview_data_%s_%s.raw', sample_name, add_name));
fprintf('Exporting data for ParaView in %s \n', filename);
fid = fopen(filename, 'w+');
fwrite(fid, data_paraview,'float');
fclose(fid);
fprintf('********************************\n');
fprintf('the tomogram size is: x = 0-%d\n', size(data_paraview, 2)-1)
fprintf('                      y = 0-%d\n', size(data_paraview, 3)-1)
fprintf('                      z = 0-%d\n', size(data_paraview, 4)-1)
fprintf('                scalars = %d\n', size(data_paraview, 1))

