% [errormetric, penalty_error, regularization_coeff]=find_regularization_mu(projection, mu, it_max, params)

%*-------------------------------------------------------------------------------------*
%|                                                                                     |
%|  Except where otherwise noted, this work is licensed under a                        |
%|  Creative Commons Attribution-NonCommercial-ShareAlike 4.0                          |
%|  International (CC BY-NC-SA 4.0) license.                                           |
%|                                                                                     |
%|  Copyright (c) 2018 by Paul Scherrer Institute (http://www.psi.ch)                  |
%|                                                                                     |
%|      Author: CXS group, PSI                                                         |
%*------------------------------------------------------------------------------------*
% You may use this code with the following provisions:
%
% If this code, or subfunctions or parts of it, is used for research in a
%   publication or if it is fully or partially rewritten for another
%   computing language the authors and institution should be acknowledged
%   in written form and additionally you should cite:
%     M. Liebi, M. Georgiadis, A. Menzel, P. Schneider, J. Kohlbrecher,
%     O. Bunk, and M. Guizar-Sicairos, “Nanostructure surveys of
%     macroscopic specimens by small-angle scattering tensor tomography,”
%     Nature 527, 349-352 (2015).   (doi:10.1038/nature16056)
% and
%     M. Liebi, M. Georgiadis, J. Kohlbrecher, M. Holler, J. Raabe, I.
%     Usov, A. Menzel, P. Schneider, O. Bunk and M. Guizar-Sicairos,
%     "Small-angle X-ray scattering tensor tomography: model of the
%     three-dimensional reciprocal-space map, reconstruction algorithm
%     and angular sampling requirements," Acta Cryst. A74, 12-24 (2018).
%     (doi:10.1107/S205327331701614X)
%
% A publication that focuses on describing features, or parameters, that
%    are already existing in the code should be first discussed with the
%    authors.
%
% This code and subroutines are part of a continuous development, they
%    are provided “as they are” without guarantees or liability on part
%    of PSI or the authors. It is the user responsibility to ensure its
%    proper use and the correctness of the results.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function  [errormetric, penalty_error, regularization_coeff]=find_regularization_mu(projection, mu, it_max, params)
% filename = sprintf('%sSASTT/%sfind_mu_%s_q%d-%d_%s.mat', params.save, ...
%     projection(1).fnames.sub_dir, projection(1).par.sample_name, projection(1).par.r_sum{1}(1),  projection(1).par.r_sum{1}(end),...
%     params.add_name);
%if exist(filename, 'file')
%    load(filename)
%else
errormetric=[];                 % Arrays to store values
penalty_error=[];               % Arrays to store values
regularization_coeff=[];        % Arrays to store values
%end
filename = sprintf('%s/regularization/', params.optimization_output_path);
if ~exist(filename, 'file')
    mkdir(filename);
end
ticnow = tic;
for kk = 1:length(mu)
    % load the previous optimization step
    coef_opt = importdata(sprintf('%s/result_%s_q%d-%d_coef_a1const_%s.mat', params.optimization_output_path, params.sample_name, projection(1).par.r_sum{1}(1), ...
        projection(1).par.r_sum{1}(end), params.add_name));
    params = coef_opt.params;
    % provide the initial guess
    params.theta_out1 = coef_opt.theta_out1;
    params.phi_out1 = coef_opt.phi_out1;
    params.a1 = coef_opt.params.a1;
    params.a_out1 = coef_opt.a_out1;
    params.itmax = it_max;              % maximum number of iteration
    params.regularization_angle_coeff = mu(kk);
    
    % optimize with the regularization
    
    % parameters for optimization of angles with the 4 coefficients
    %%% Vector with spherical harmonic parameters
    l = [0  2 4 6];  % Polar order
    m = [0  0 0 0];  % Azimuthal order
    a = [0.001  -0.0001 0.00001 -0.000001];   % Coefficients - not used
    params.l = l;                % polar order of spherical harmonics
    params.m = m;                % azimuthal order of spherical harmonics
    %%% Initial conditions (for each voxel in the tomogram)
    theta_0_init = reshape(params.theta_out1,1,[]);
    phi_0_init   = reshape(params.phi_out1,1,[]);
    a_init= params.a_out1;
    a_init = reshape(a_init,1,[]);

    numOfvoxels = params.numOfvoxels;

    params.coeff_soft_limits_low = -Inf;
    params.coeff_soft_limits_high = Inf;
    %parameter for the penalty for the soft limits of the coefficient
    params.soft_limit_weight_coeff=1e5;

    params.find_orientation = 1;    % Optimize over the main orientation of the structure
    params.init_guess = 0;          % in case we use initial guesses
    params.find_coefficients = 1;   % Optimize over coefficients
    params.avoid_wrapping = 1;      % avoid wrapping over 2Pi of the angle
    params.a1_const=0;              %keeping first coeff constnat (true false)
    params.regularization = 0;      % Sieves regularization on coeff
    params.regularization_angle = 1;%regularization of angle

    % new feature to show error during calculation
    params.display = 1; %display intermediate results from optimization
    params.dispinter = 1; % number of iterations step results are displayed
    params.plot = 1; %plot intermediate results
    %parameters for the projection at arbitary angles
    params.volume_upsampling = 1;  % Upsamples the volume by a factor of 2 to reduce artifacts
    params.method = 'bilinear';    % 'nearest' or 'bilinear' method for volume allocation
    params.filter_2D = 1;          % Strength of filter applied to image
    %parameters for the sieves regularization (blurring the gradient)
    kernel3D=window3(5,5,5,@hamming);
    params.kernel=kernel3D./sum(kernel3D(:)); % for normalization (sum equals 1)

    %optimize
    [params, theta_out1, phi_out1, a_out1] = ...
        optimization.optimize_SH(projection, params, theta_0_init, phi_0_init, a_init);

    % save results
    % load the result from cgmin1
    load(sprintf('%s/current_opt_saxs_all.mat', params.optimization_output_path));
    filename = sprintf('%s/regularization/result_%s_q%d-%d_lcurve_%s_reg_%.0e.mat', params.optimization_output_path, params.sample_name, projection(1).par.r_sum{1}(1),  projection(1).par.r_sum{1}(end),...
        params.add_name, params.regularization_angle_coeff);
    save(filename,'a_out1','theta_out1','phi_out1','e', 'params', '-v6');
    fprintf('Saving data to: %s \n', filename)

    if params.regularization_angle_coeff == mu(end)
        filename = sprintf('%s/regularization/optimization_lcurve_%s_reg_%.0e.jpg', params.optimization_output_path, params.sample_name,...
            params.regularization_angle_coeff);
        print(gcf, filename,'-djpeg','-r300');
        fprintf('Saving figure to: %s \n', filename)
    end
    
    
    
    % load the optimization
    filename = sprintf('%s/regularization/result_%s_q%d-%d_lcurve_%s_reg_%.0e.mat', params.optimization_output_path, params.sample_name, projection(1).par.r_sum{1}(1),  projection(1).par.r_sum{1}(end),...
        params.add_name, params.regularization_angle_coeff);
    load(filename)
    %plot_3D_color(params, slice_nr, filename)
    fig_save = plot_3D_color(params, [], filename);
    filename = sprintf('%s/regularization/result_%s_q%d-%d_lcurve_%s_reg_%.0e.jpg', params.optimization_output_path, params.sample_name, projection(1).par.r_sum{1}(1),  projection(1).par.r_sum{1}(end),...
        params.add_name, params.regularization_angle_coeff);
    print(fig_save, filename,'-djpeg','-r300');
    % load the parameters from the optimization
    params.theta_0 = theta_out1;
    params.phi_0 = phi_out1;
    params.a = a_out1;
    % calculate the projections
    [proj_out,E,Reg,Ereg] = plot_projection_reg(params, projection(1:p.skip_projections:end));
    % save the calculated error
    errormetric = [errormetric,E];
    penalty_error  =[penalty_error,Ereg];
    regularization_coeff = [regularization_coeff,params.regularization_angle_coeff];
end
toc(ticnow);

% plot L-curve
figure(40);
subplot(1, 2, 1)
hold on
plot(errormetric,(penalty_error./regularization_coeff),'ko--')
xlabel('error')
ylabel('regularization error')
ytickformat('%.2f')
xtickformat('%.2f')
% plot stabilization parameter (Glatter)
fig = subplot(1, 2, 2);
%hold on
left_color = [0 0 0];
right_color = [1 0 0];
set(fig,'defaultAxesColorOrder',[left_color; right_color]);
yyaxis left
semilogx(regularization_coeff,penalty_error./regularization_coeff,'ko--')
grid on
xlabel('regularization parameter \mu')
ylabel('regularization error')
ytickformat('%.2f')
hold on
yyaxis right
semilogx(regularization_coeff,errormetric,'ro--')
grid on
ylabel('error')
ytickformat('%.2f')

%% save
filename = sprintf('%s/regularization/find_mu_%s_q%d-%d_%s.mat', params.optimization_output_path, params.sample_name, projection(1).par.r_sum{1}(1),  projection(1).par.r_sum{1}(end),...
    params.add_name);
fprintf('Saving to: %s \n', filename)
save(filename,'errormetric', 'penalty_error', 'regularization_coeff', 'params', '-v6');

filename = sprintf('%s/regularization/find_mu_%s_q%d-%d_%s.jpg', params.optimization_output_path, params.sample_name, projection(1).par.r_sum{1}(1),  projection(1).par.r_sum{1}(end),...
    params.add_name);
fprintf('Saving figure to: %s \n', filename)
print(gcf, filename,'-djpeg','-r300');